#include "ch81.h"
#include "solver.h"
#include "rowminlex.h"

#include <malloc.h>

void doPlus1Minimal(puzzleSet &puzzles, puzzleSet &resSS, puzzleSet &resMS) {
	size_t nPuzzles = puzzles.size();
	const ch81 ** const index = (const ch81 **)malloc(nPuzzles * sizeof(ch81 *));
	nPuzzles = 0;
	for(puzzleSet::iterator p = puzzles.begin(); p != puzzles.end(); p++) {
		index[nPuzzles++] = &(*p);
	}
	//suitable for parallelization
#ifdef _OPENMP
#pragma omp parallel for schedule(dynamic, 100)
#endif //_OPENMP
	for(int i = 0; i < nPuzzles; i++) {
		char p1[81 * 81 * 9];
		subCanoner sc;
		int p1Size = solverPlus1(index[i]->chars, p1, true, false);
		if(p1Size) {
			puzzleSet ss;
			puzzleSet ms;
			//split uniques from non-uniques
			for(int j = 0; j < p1Size; j++) {
				ch81 pcan;
				sc.canon(&p1[81 * j], pcan.chars);
				if(1 == solve(pcan.chars, 2)) {
					//single solution
					ss.insert(pcan);
				}
				else {
					//multiple solutions
					ms.insert(pcan);
				}
			}
#ifdef _OPENMP
#pragma omp critical
#endif //_OPENMP
			{
				resMS.insert(ms.begin(), ms.end());
				resSS.insert(ss.begin(), ss.end());
			}
		}
	}
	free(index);
	puzzles.clear();
}

extern int doPlus1() {
	puzzleSet puzzles;
	puzzleSet resSS;
	puzzleSet resMS;
	puzzles.loadFromFile(stdin); // always canonicalize the input
	doPlus1Minimal(puzzles, resSS, resMS); //result is returned in the place of source
	resSS.saveToFile(stdout);
	resMS.saveToFile(stderr);
	return 0;
}
